/*
 * Szegedi Tudományegyetem
 * Informatikai Tanszékcsoport
 * Szoftverfejlesztés Tanszék
 */

#include "pnm-manipulation.h"

#include <malloc.h>

#define FORALLPIXELS(P,I) for (unsigned int I = 0, __pxcnt = P->width * P->height; I < __pxcnt; ++I)

void rgbpic_invert(rgbpic_t *pic) {
    FORALLPIXELS(pic, i) {
        pic->pixelvalues[i].red   = pic->maxpixvalue - pic->pixelvalues[i].red;
        pic->pixelvalues[i].green = pic->maxpixvalue - pic->pixelvalues[i].green;
        pic->pixelvalues[i].blue  = pic->maxpixvalue - pic->pixelvalues[i].blue;
    }
}

void rgbpic_invert_R(rgbpic_t *pic) {
    FORALLPIXELS(pic, i) {
        pic->pixelvalues[i].red   = pic->maxpixvalue - pic->pixelvalues[i].red;
    }
}

void rgbpic_invert_G(rgbpic_t *pic) {
    FORALLPIXELS(pic, i) {
        pic->pixelvalues[i].green = pic->maxpixvalue - pic->pixelvalues[i].green;
    }
}

void rgbpic_invert_B(rgbpic_t *pic) {
    FORALLPIXELS(pic, i) {
        pic->pixelvalues[i].blue  = pic->maxpixvalue - pic->pixelvalues[i].blue;
    }
}

void rgbpic_mirror_h(rgbpic_t *pic) {
    int i, j, l, r;
    rgbpic_pixel_t temp;
    for (i = 0; i < pic->height; ++i) {
        for (j = 0; j < pic->width / 2; ++j) {
            l = rgbpic_coord(pic, j, i);
            r = rgbpic_coord(pic, pic->width - j - 1, i);
            temp                = pic->pixelvalues[l];
            pic->pixelvalues[l] = pic->pixelvalues[r];
            pic->pixelvalues[r] = temp;
        }
    }
}

void rgbpic_mirror_v(rgbpic_t *pic) {
    int i, j, l, r;
    rgbpic_pixel_t temp;
    for (j = 0; j < pic->width; ++j) {
        for (i = 0; i < pic->height / 2; ++i) {
            l = rgbpic_coord(pic, j, i);
            r = rgbpic_coord(pic, j, pic->height - i - 1);
            temp                = pic->pixelvalues[l];
            pic->pixelvalues[l] = pic->pixelvalues[r];
            pic->pixelvalues[r] = temp;
        }
    }
}

void rgbpic_rotate_l(rgbpic_t *pic) {
    int i, j, n;
    rgbpic_pixel_t *temp;
    n = pic->width * pic->height;
    temp = (rgbpic_pixel_t*)malloc(n * sizeof(rgbpic_pixel_t));
    if (!temp) {
        return;
    }
    for (i = 0; i < pic->height; ++i) {
        for (j = 0; j < pic->width; ++j) {
            temp[(pic->width - 1 - j) * pic->height + i] =
                    pic->pixelvalues[rgbpic_coord(pic, j, i)];
        }
    }
    n           = pic->height;
    pic->height = pic->width;
    pic->width  = n;
    free(pic->pixelvalues);
    pic->pixelvalues = temp;
}

void rgbpic_mirror_c(rgbpic_t *pic) {
    int i, n;
    rgbpic_pixel_t temp;
    n = pic->width * pic->height;
    for (i = 0; i < n / 2; ++i) {
        temp                        = pic->pixelvalues[i];
        pic->pixelvalues[i]         = pic->pixelvalues[n - i - 1];
        pic->pixelvalues[n - i - 1] = temp;
    }
}

void rgbpic_rotate_r(rgbpic_t *pic) {
    int i, j, n;
    rgbpic_pixel_t *temp;
    n = pic->width * pic->height;
    temp = (rgbpic_pixel_t*)malloc(n * sizeof(rgbpic_pixel_t));
    if (!temp) {
        return;
    }
    for (i = 0; i < pic->height; ++i) {
        for (j = 0; j < pic->width; ++j) {
            temp[j * pic->height + pic->height - 1 - i] =
                    pic->pixelvalues[rgbpic_coord(pic, j, i)];
        }
    }
    n           = pic->height;
    pic->height = pic->width;
    pic->width  = n;
    free(pic->pixelvalues);
    pic->pixelvalues = temp;
}

void rgbpic_color_RG(rgbpic_t *pic) {
    FORALLPIXELS(pic, i) {
        rgbpic_monochrome_pixel_t m;
        m                         = pic->pixelvalues[i].red;
        pic->pixelvalues[i].red   = pic->pixelvalues[i].green;
        pic->pixelvalues[i].green = m;
    }
}

void rgbpic_color_GB(rgbpic_t *pic) {
    FORALLPIXELS(pic, i) {
        rgbpic_monochrome_pixel_t m;
        m                         = pic->pixelvalues[i].green;
        pic->pixelvalues[i].green = pic->pixelvalues[i].blue;
        pic->pixelvalues[i].blue  = m;
    }
}

void rgbpic_color_BR(rgbpic_t *pic) {
    FORALLPIXELS(pic, i) {
        rgbpic_monochrome_pixel_t m;
        m                        = pic->pixelvalues[i].blue;
        pic->pixelvalues[i].blue = pic->pixelvalues[i].red;
        pic->pixelvalues[i].red  = m;
    }
}

void rgbpic_delete_R(rgbpic_t *pic) {
    FORALLPIXELS(pic, i) {
        pic->pixelvalues[i].red = 0;
    }
}

void rgbpic_delete_G(rgbpic_t *pic) {
    FORALLPIXELS(pic, i) {
        pic->pixelvalues[i].green = 0;
    }
}

void rgbpic_delete_B(rgbpic_t *pic) {
    FORALLPIXELS(pic, i) {
        pic->pixelvalues[i].blue = 0;
    }
}

void rgbpic_convert_to_grey_avg(rgbpic_t *pic) {
    rgbpic_monochrome_pixel_t v;
    FORALLPIXELS(pic, i) {
        v = (pic->pixelvalues[i].red + pic->pixelvalues[i].green + pic->pixelvalues[i].blue) / 3;
        pic->pixelvalues[i].red   = v;
        pic->pixelvalues[i].green = v;
        pic->pixelvalues[i].blue  = v;
    }
}

void rgbpic_convert_to_grey_lum(rgbpic_t *pic) {
    rgbpic_monochrome_pixel_t v;
    FORALLPIXELS(pic, i) {
        v = rgbpic_rgb_to_gray(pic->pixelvalues[i]);
        pic->pixelvalues[i].red   = v;
        pic->pixelvalues[i].green = v;
        pic->pixelvalues[i].blue  = v;
    }
}

void rgbpic_convert_R_to_grey(rgbpic_t *pic) {
    FORALLPIXELS(pic, i) {
        pic->pixelvalues[i].green = pic->pixelvalues[i].red;
        pic->pixelvalues[i].blue  = pic->pixelvalues[i].red;
    }
}

void rgbpic_convert_G_to_grey(rgbpic_t *pic) {
    FORALLPIXELS(pic, i) {
        pic->pixelvalues[i].red  = pic->pixelvalues[i].green;
        pic->pixelvalues[i].blue = pic->pixelvalues[i].green;
    }
}

void rgbpic_convert_B_to_grey(rgbpic_t *pic) {
    FORALLPIXELS(pic, i) {
        pic->pixelvalues[i].red   = pic->pixelvalues[i].blue;
        pic->pixelvalues[i].green = pic->pixelvalues[i].blue;
    }
}

void rgbpic_convert_to_bw_avg(rgbpic_t *pic) {
    rgbpic_monochrome_pixel_t v;
    FORALLPIXELS(pic, i) {
        v = (2 * (pic->pixelvalues[i].red + pic->pixelvalues[i].green + pic->pixelvalues[i].blue) > 3 * pic->maxpixvalue) * pic->maxpixvalue;
        pic->pixelvalues[i].red   = v;
        pic->pixelvalues[i].green = v;
        pic->pixelvalues[i].blue  = v;
    }
}

void rgbpic_convert_to_bw_lum(rgbpic_t *pic) {
    rgbpic_monochrome_pixel_t v;
    FORALLPIXELS(pic, i) {
        v = (rgbpic_rgb_to_gray(pic->pixelvalues[i]) > pic->maxpixvalue / 2) * pic->maxpixvalue;
        pic->pixelvalues[i].red   = v;
        pic->pixelvalues[i].green = v;
        pic->pixelvalues[i].blue  = v;
    }
}
